<?php
namespace AJAXGuestBook\controller;

use AJAXGuestBook\core\{Auth, Badwords, Controller, GBLocale, Session, Paginator, SpamBook};

class Admin extends Controller {
    function __construct(){
        parent::__construct();
        $this->title .= " - Administrator";
        Auth::handleLogin();
        $this->view->addGlobals('isdemo', GBDEMO);
    }

    public function index($param = null){
        if(isset($param)){
            $this->errorpage();
            return;
        }
        $totalEntries = $this->guestbook->getTotalEntries();
        $latestEntries = $this->guestbook->loadEntries(0,5,false);
        $pendingApproval = $this->guestbook->getUnapprovedEntries();
        $latestPending = $this->guestbook->loadEntries(0,5,false,false,"false");
        $spambook = SpamBook::getInstance();
        $spamcount = $spambook->getTotalEntries();
        echo $this->view->render('admin/index.html', ['title' => $this->title, 'totalcount' => $totalEntries,
                                                      'pendingcount' => $pendingApproval, 'spamcount' => $spamcount,
                                                      'latestentries' => $latestEntries, 'latestpending' => $latestPending]);
    }

    public function general($action = null, $param = null){
        if(isset($action) && GBDEMO){
            Session::set('gberror', 'Not working in Demo mode.');
        }
        if(isset($param)){
            $this->errorpage();
            return;
        }
        if(isset($action) && !GBDEMO){
            switch($action){
                case "update":
                    $adminModel = $this->loadModel('AdminModel', true);
                    $result = $adminModel->updateGeneralSettings();
                    if(isset($result['error'])){
                        Session::set("gberror", $result['error']);
                    } else if(isset($result['success'])){
                        Session::set("gbsuccess", $result['success']);
                    }
                    break;
                default:
                    $this->errorpage();
                    break;
            }
        }
        echo $this->view->render('admin/general.html', ['title' => $this->title,
                                                        'gboptions' => $this->guestbook->getSettings(),
                                                        'post' => $_POST]);
    }

    public function profile($action = null, $param = null){
        if(isset($action) && GBDEMO){
            Session::set('gberror', 'Not working in Demo mode.');
        }
        if(isset($param)){
            $this->errorpage();
            return;
        }
        if(isset($action) && !GBDEMO){
            switch($action){
                case "update":
                    $adminModel = $this->loadModel('AdminModel', true);
                    $result = $adminModel->updateProfileSettings();
                    if(isset($result['error'])){
                        Session::set("gberror", $result['error']);
                    } else if(isset($result['success'])){
                        Session::set("gbsuccess", $result['success']);
                    }
                    break;
                default:
                    $this->errorpage();
                    break;
            }
        }
        echo $this->view->render('admin/profile.html', ['title' => $this->title,
                                                        'gboptions' => $this->guestbook->getSettings(),
                                                        'post' => $_POST]);
    }

    public function guestbook($action = null, $param = null){
        if(isset($action)){
            switch($action){
                case "edit":
                    if(isset($param)){
                        $this->setSessionReturnUrl('/admin/guestbook/');
                        $result = $this->guestbook->loadGBEntry($param);
                        if(isset($result['error'])){
                            Session::set("gberror", $result['error']);
                            header('Location: ' . urldecode(Session::getanddelete("returnurl")));
                            exit();
                        } else{
                            $commentinfo = json_decode($result['data']);
                            Session::set('commentinfo', $commentinfo);
                            echo $this->view->render('admin/editentry.html', ['title' => $this->title,
                                                                              'gboptions' => $this->guestbook->getSettings(),
                                                                              'post' => $_POST,
                                                                              'commentinfo' => $commentinfo]);
                        }
                    } else{
                        $this->errorpage();
                    }
                    break;
                case "update":
                    $result = $this->guestbook->updateGBEntry();
                    if(isset($result['error'])){
                        Session::set("gberror", $result['error']);
                        $commentinfo = Session::get('commentinfo');
                        echo $this->view->render('admin/editentry.html', ['title' => $this->title,
                                                                          'gboptions' => $this->guestbook->getSettings(),
                                                                          'post' => $_POST,
                                                                          'commentinfo' => $commentinfo]);
                    } else if(isset($result['success'])){
                        Session::delete('commentinfo');
                        Session::set("gbsuccess", $result['success']);
                        header("Location: " . $_POST ['returnurl']);
                        exit;
                    }
                    break;
                case "delete":
                    if(isset($_POST['submit'])){
                        $this->setSessionReturnUrl('/admin/guestbook/');
                        if(isset($_POST['cbmid'])){
                            $result = $this->guestbook->deleteMultipleGBEntries();
                            if(isset($result['error'])){
                                Session::set("gberror", $result['error']);
                            } else if(isset($result['success'])){
                                Session::set("gbsuccess", $result['success']);
                            }
                        } else{
                            Session::set("gberror", "No entries selected for deletion.");
                        }
                        header('Location: ' . urldecode(Session::getanddelete("returnurl")));
                        exit;
                    } else if(isset($param)){
                        $this->setSessionReturnUrl('/admin/guestbook/');
                        $result = $this->guestbook->loadGBEntry($param);
                        if(isset($result['error'])){
                            Session::set("gberror", $result['error']);
                            header('Location: ' . urldecode(Session::getanddelete("returnurl")));
                            exit();
                        } else{
                            $commentinfo = json_decode($result['data']);
                            Session::set('commentinfo', $commentinfo);
                            echo $this->view->render('admin/deleteentry.html', ['title' => $this->title,
                                                                                'gboptions' => $this->guestbook->getSettings(),
                                                                                'post' => $_POST,
                                                                                'commentinfo' => $commentinfo]);
                        }
                    } else{
                        $this->errorpage();
                    }
                    break;
                case "confirmdelete":
                    $result = $this->guestbook->deleteGBEntry();
                    if(isset($result['error'])){
                        Session::set("gberror", $result['error']);
                        $commentinfo = Session::get('commentinfo');
                        echo $this->view->render('admin/deleteentry.html', ['title' => $this->title,
                                                                            'gboptions' => $this->guestbook->getSettings(),
                                                                            'post' => $_POST,
                                                                            'commentinfo' => $commentinfo]);
                    } else if(isset($result['success'])){
                        Session::delete('commentinfo');
                        Session::set("gbsuccess", $result['success']);
                        header("Location: " . $_POST ['returnurl']);
                        exit;
                    }
                    break;
                case "approve":
                    if(isset($param)){
                        $this->setSessionReturnUrl('/admin/guestbook/');
                        $result = $this->guestbook->approveEntry($param);
                        if(isset($result['error'])){
                            Session::set("gberror", $result['error']);
                        } else{
                            Session::set("gbsuccess", $result['success']);
                        }
                        header('Location: ' . urldecode(Session::getanddelete("returnurl")));
                        exit();
                    } else{
                        $this->errorpage();
                    }
                    break;
                case "updatexml":
                    $result = $this->guestbook->updateXMLDoc();
                    if(isset($result['error'])){
                        Session::set("gberror", $result['error']);
                    } else if(isset($result['success'])){
                        Session::set("gbsuccess", $result['success']);
                    }
                    header("Location: " . URL . '/admin/guestbook/');
                    exit;
                    break;
                default:
                    $this->errorpage();
                    break;
            }
        } else{
            $this->view->addTwigFunction('getxmldatabase', function(){
                return file_get_contents(DATAPATH . DS . $this->guestbook->database);
            }, ['is_safe' => ['html']]);
            $paginator = new Paginator;
            $paginator->items_total = $this->guestbook->getTotalEntries();
            $paginator->mid_range = 5; // Number of pages to display. Must be odd and > 3
            $paginator->default_ipp = $this->guestbook->entriesPerPage;
            $paginator->paginate(URL . "/admin/guestbook/");
            if(isset($_GET['ipp']) and $_GET['ipp'] == 'All'){
                $messages = $this->guestbook->loadEntries(0, 0, true, true);
            } else{
                $messages = $this->guestbook->loadEntries($paginator->low, $paginator->items_per_page, false, true);
            }
            echo $this->view->render('admin/entries.html', ['title' => $this->title,
                                                            'gboptions' => $this->guestbook->getSettings(),
                                                            'minMessage' => $this->guestbook->minCommentsChars,
                                                            'maxMessage' => $this->guestbook->maxCommentsChars,
                                                            'emcodes' => json_encode($this->guestbook->emoticon->getEmoticonCodeList()),
                                                            'emfiles' => json_encode($this->guestbook->emoticon->getEmoticonFileList()),
                                                            'smileys' => $this->guestbook->emoticon->getEmoticonSrcList(),
                                                            'messages' => $messages,
                                                            'paginator' => $paginator->page_data(),
                                                            'querystring' => parse_url($_SERVER['REQUEST_URI'], PHP_URL_QUERY),
                                                            'requiredfields' => $this->guestbook->requiredFields,
                                                            'captcha' => $this->guestbook->captcha, 'post' => $_POST]);
        }
    }

    public function locale($action = null, $param = null){
        if(isset($action) && GBDEMO){
            Session::set('gberror', 'Not working in Demo mode.');
        }
        if(isset($param)){
            $this->errorpage();
            return;
        }
        if(isset($action) && !GBDEMO){
            switch($action){
                case "update":
                    if(isset($_POST["updatelocale"])){
                        $vals["timezone"] = $_POST["timezone"];
                        $vals["language"] = $_POST["language"];
                        $result = $this->guestbook->saveLocaleSettings($vals);
                        if(isset($result['error'])){
                            Session::set("gberror", $result['error']);
                        } else if(isset($result['success'])){
                            Session::set("gbsuccess", $result['success']);
                        }
                    }elseif(isset($_POST["updatetheme"])){
                        $vals["theme"] = $_POST["theme"];
                        $result = $this->guestbook->saveThemeSettings($vals);
                        if(isset($result['error'])){
                            Session::set("gberror", $result['error']);
                        } else if(isset($result['success'])){
                            Session::set("gbsuccess", $result['success']);
                        }
                    }
                    break;
                default:
                    $this->errorpage();
                    return;
            }
        }

        $themelist = getThemeList();

        echo $this->view->render('admin/locale.html', ['title' => $this->title,
                                                       'gboptions' => $this->guestbook->getSettings(),
                                                       'timezones' => GBLocale::getTimeZoneList(),
                                                       'languages' => GBLocale::getCarbonLocales(),
                                                       'themelist' => $themelist,
                                                       'post' => $_POST,]);
    }

    public function spam($action = null, $param = null){
        if(isset($action) && GBDEMO){
            Session::set('gberror', 'Not working in Demo mode.');
        }
        $spambook = SpamBook::getInstance();
        $this->view->addTwigFunction('getxmldatabase', function() use ($spambook){
            return file_get_contents(DATAPATH . DS . $spambook->database);
        }, ['is_safe' => ['html']]);
        $paginator = new Paginator;
        $paginator->items_total = $spambook->getTotalEntries();
        $paginator->mid_range = 5; // Number of pages to display. Must be odd and > 3
        $paginator->default_ipp = $spambook->entriesPerPage;
        $paginator->paginate(URL . "/admin/spam/");
        if(isset($_GET['ipp']) and $_GET['ipp'] == 'All'){
            $messages = $spambook->loadEntries(0, 0, true, true);
        } else{
            $messages = $spambook->loadEntries($paginator->low, $paginator->items_per_page, false, true);
        }
        if(isset($action) && !GBDEMO){
            switch($action){
                case "update":
                    $adminModel = $this->loadModel('AdminModel', true);
                    $result = $adminModel->updateSpamSettings();
                    if(isset($result['error'])){
                        Session::set("gberror", $result['error']);
                    } else if(isset($result['success'])){
                        Session::set("gbsuccess", $result['success']);
                    }
                    echo $this->view->render('admin/spam.html', ['title' => $this->title,
                                                                 'gboptions' => $this->guestbook->getSettings(),
                                                                 'minMessage' => $this->guestbook->minCommentsChars,
                                                                 'maxMessage' => $this->guestbook->maxCommentsChars,
                                                                 'emcodes' => json_encode($this->guestbook->emoticon->getEmoticonCodeList()),
                                                                 'emfiles' => json_encode($this->guestbook->emoticon->getEmoticonFileList()),
                                                                 'smileys' => $this->guestbook->emoticon->getEmoticonSrcList(),
                                                                 'messages' => $messages,
                                                                 'paginator' => $paginator->page_data(),
                                                                 'querystring' => parse_url($_SERVER['REQUEST_URI'], PHP_URL_QUERY),
                                                                 'post' => $_POST]);
                    break;
                case "approve":
                    if(isset($param)){
                        $this->setSessionReturnUrl('/admin/spam/');
                        $result = $spambook->approveSpamEntry($param);
                        if(isset($result['error'])){
                            Session::set("gberror", $result['error']);
                        } else if(isset($result['success'])){
                            Session::set("gbsuccess", $result['success']);
                        }
                        header('Location: ' . urldecode(Session::getanddelete("returnurl")));
                        exit();
                    } else{
                        $this->errorpage();
                    }
                    break;
                case "delete":
                    if(isset($_POST['submit'])){
                        $this->setSessionReturnUrl('/admin/spam/');
                        if(isset($_POST['cbmid'])){
                            $result = $spambook->deleteMultipleGBEntries();
                            if(isset($result['error'])){
                                Session::set("gberror", $result['error']);
                            } else if(isset($result['success'])){
                                Session::set("gbsuccess", $result['success']);
                            }
                        } else{
                            Session::set("gberror", "No entries selected for deletion.");
                        }
                        header('Location: ' . urldecode(Session::getanddelete("returnurl")));
                        exit;
                    } else if(isset($param)){
                        $this->setSessionReturnUrl('/admin/spam/');
                        $result = $spambook->deleteSpamEntry($param);
                        if(isset($result['error'])){
                            Session::set("gberror", $result['error']);
                        } else if(isset($result['success'])){
                            Session::set("gbsuccess", $result['success']);
                        }
                        header("Location: " . urldecode(Session::getanddelete("returnurl")));
                        exit;
                    } else{
                        $this->errorpage();
                    }
                    break;
                case "updatexml":
                    $result = $spambook->updateXMLDoc();
                    if(isset($result['error'])){
                        Session::set("gberror", $result['error']);
                    } else if(isset($result['success'])){
                        Session::set("gbsuccess", $result['success']);
                    }
                    header("Location: " . URL . '/admin/spam/');
                    exit;
                    break;
                default:
                    $this->errorpage();
                    break;
            }
        } else{
            echo $this->view->render('admin/spam.html', ['title' => $this->title,
                                                         'gboptions' => $this->guestbook->getSettings(),
                                                         'minMessage' => $this->guestbook->minCommentsChars,
                                                         'maxMessage' => $this->guestbook->maxCommentsChars,
                                                         'emcodes' => json_encode($this->guestbook->emoticon->getEmoticonCodeList()),
                                                         'emfiles' => json_encode($this->guestbook->emoticon->getEmoticonFileList()),
                                                         'smileys' => $this->guestbook->emoticon->getEmoticonSrcList(),
                                                         'messages' => $messages,
                                                         'paginator' => $paginator->page_data(),
                                                         'querystring' => parse_url($_SERVER['REQUEST_URI'], PHP_URL_QUERY),
                                                         'post' => $_POST]);
        }
    }

    public function badwords($action = null, $param = null){
        if(isset($param)){
            $this->errorpage();
            return;
        }
        if(isset($action) && GBDEMO){
            Session::set('gberror', 'Not working in Demo mode.');
        }
        $result = [];
        if(isset($action) && !GBDEMO){
            switch($action){
                case "update":
                    if(isset($_POST['submit'])){
                        $result = Badwords::saveBadwordsList($this->guestbook->badWordsFile);
                        if(isset($result['error'])){
                            Session::set("gberror", $result['error']);
                        } else if(isset($result['success'])){
                            Session::set("gbsuccess", $result['success']);
                        }
                    }
                    break;
                default:
                    $this->errorpage();
                    break;
            }
        } else{
            $result = Badwords::loadBadwordsList($this->guestbook->badWordsFile);
            if(isset($result['error'])){
                Session::set("gberror", $result['error']);
            }
        }
        echo $this->view->render('admin/badwords.html', ['title' => $this->title,
                                                         'gboptions' => $this->guestbook->getSettings(),
                                                         'bwlist' => $result['data']]);
    }

    private function setSessionReturnUrl($url){
        if(isset($_SERVER["HTTP_REFERER"]) && $_SERVER["HTTP_REFERER"] !== ''){
            Session::set("returnurl", urlencode($_SERVER["HTTP_REFERER"]));
        } else{
            Session::set("returnurl", urlencode(URL . $url));
        }
    }
}