<?php
namespace AJAXGuestBook\controller;

use AJAXGuestBook\core\GBLocale;
use AJAXGuestBook\core\Session;
use AJAXGuestBook\core\View;
use AJAXGuestBook\models\InstallModel;

class Install {
    public $title = "AJAX Guestbook";
    public $navtitle = "AJAX Guestbook";
    protected $view;

    function __construct(){
        if(is_dir(DATAPATH)){
            if(file_exists(DATAPATH . DS . "install.lock")){
                exit("<h3>Installation is locked. Please remove the lock file to install the guestbook.</h3>");
            }
        }
        $this->view = new View();
        $this->view->addGlobals('website', "http://www.aiglemedia.com");
        $this->view->addGlobals('company', "AigleMedia");
        $this->view->addGlobals('title', $this->title);
        $this->view->addGlobals('navtitle', $this->title);
    }

    private function trim_value(&$value){
        $value = trim($value);
    }

    public function index(){
        $php['version'] = PHP_VERSION;
        if(version_compare(PHP_VERSION, '7.1.29', '<')){
            $php['error'] = true;
            $php['message'] = 'Requires PHP version 7.1.29 or above.';
        } else{
            $php['message'] = 'PHP version 7.1.29 or above is installed.';
        }
        $version = $_SERVER['SERVER_SOFTWARE'];
        $apache['version'] = substr($version, 0, strpos($version, " "));
        if(strpos($version, 'Apache') === false){
            $apache['error'] = true;
            $apache['message'] = 'Apache web server required.';
        } else{
            $apache['message'] = 'Apache web server running.';
        }
        if(!extension_loaded("gd")){
            $gderror = true;
        } else{
            $gderror = false;
        }
        if(!extension_loaded("mbstring")){
            $mbstringerror = true;
        } else{
            $mbstringerror = false;
        }
        $fdcheck['message'] = 'Directory and File permissions ok.';
        if(is_dir(DATAPATH)){
            if(!is_writable(DATAPATH)){
                if(!chmod(DATAPATH, 0755)){
                    $fdcheck['error'] = true;
                    $fdcheck['message'] = 'Unable to set proper directory permissions.';
                }
            }
            if(file_exists(DATAPATH . "/settings.xml")){
                $fdcheck['fileexists'] = true;
            }
        } else{
            if(!mkdir(DATAPATH)){
                $fdcheck['error'] = true;
                $fdcheck['message'] = 'Unable to create directory for database storage at "' . ROOTPATH . '" path';
            }
        }
        echo $this->view->render('install/index.html', ['php' => $php, 'apache' => $apache, 'gderror' => $gderror,
                                                        'mbstringerror' => $mbstringerror, 'fdcheck' => $fdcheck]);
    }

    public function wizard($action = null){
        array_filter($_POST, array($this, 'trim_value'));
        if(isset($_POST["task"])){
            $task = filter_input(INPUT_POST, "task", FILTER_SANITIZE_STRING);
            $insallmodel = new InstallModel();
            switch($task){
                case "validategeneral":
                    if($insallmodel->validateGeneralSettings()){
                        header('Location: ' . URL . '/install/wizard/step2');
                        exit();
                    } else{
                        Session::set('gberror', 'Please rectify the errors shown below.');
                    }
                    break;
                case "validateprofile":
                    if($insallmodel->validateProfileSettings()){
                        header('Location: ' . URL . '/install/wizard/step3');
                        exit();
                    } else{
                        Session::set('gberror', 'Please rectify the errors shown below.');
                    }
                    break;
                case "validatelocale":
                    if($insallmodel->validateLocaleSettings()){
                        header('Location: ' . URL . '/install/wizard/confirm');
                        exit();
                    } else{
                        Session::set('gberror', 'Please rectify the errors shown below.');
                    }
                    break;
                case "installsettings":
                    if($insallmodel->installSettings()){
                        header('Location: ' . URL . '/home/welcome');
                        exit();
                    }
                    break;
                default:
                    $this->errorpage();
                    break;
            }
        }
        if(isset($action)){
            $step = trim($action);
            switch($step){
                case "step1":
                    if(!Session::get('gbrequirements')) header('Location: ' . URL . '/install/');
                    echo $this->view->render('install/general.html', ['installgeneral' => Session::get('installgeneral'),
                                                                      'post' => $_POST]);
                    break;
                case "step2":
                    if(!Session::get('installgeneral')) header('Location: ' . URL . '/install/wizard/step1');
                    echo $this->view->render('install/profile.html', ['installprofile' => Session::get('installprofile'),
                                                                      'post' => $_POST]);
                    break;
                case "step3":
                    if(!Session::get('installprofile')) header('Location: ' . URL . '/install/wizard/step2');
                    echo $this->view->render('install/locale.html', ['installlocale' => Session::get('installlocale'),
                                                                     'timezones' => GBLocale::getTimeZoneList(),
                                                                     'languages' => GBLocale::getCarbonLocales(),
                                                                     'post' => $_POST]);
                    break;
                case "confirm":
                    if(!Session::get('installlocale')) header('Location: ' . URL . '/install/wizard/step3');
                    echo $this->view->render('install/confirm.html', ['installgeneral' => Session::get('installgeneral'),
                                                                      'installprofile' => Session::get('installprofile'),
                                                                      'installlocale' => Session::get('installlocale'),
                                                                      'languages' => GBLocale::getCarbonLocales(),
                                                                      'post' => $_POST]);
                    break;
                default:
                    $this->errorpage();
                    break;
            }
        } else{
            $this->errorpage();
            exit();
        }
    }

    public function errorpage(){
        $this->title .= " - Page Not Found";
        header('HTTP/1.0 404 Not Found');
        echo $this->view->render('404.html', ['title' => $this->title]);
    }
}