<?php
namespace AJAXGuestBook\core;

use DOMXPath;

/**
 * @property \DOMDocument $xmlDoc
 * @property Emoticon $emoticon
 */
trait CommonMethods {
    /*Generate random unique id for message.*/
    private function getRandomUniqueID(){
        do{
            $uniqueid = uniqid("mid");
        } while($this->checkIfUniqueIdExists($uniqueid));
        return $uniqueid;
    }

    private function checkIfUniqueIdExists($uid){
        $messageList = new DOMXpath($this->xmlDoc);
        $uniqid = $messageList->query("/guestbook/message[@uid='" . $uid . "']");
        if($uniqid->length > 0){
            return true;
        } else{
            return false;
        }
    }

    /* Saving the xml document.*/
    private function saveXMLDoc($errorstring = null, $successstring = null){
        if(!file_exists(DATAPATH . DS . $this->database)){
            $returnvalue['error'] = "Error opening the xml database file.";
            return $returnvalue;
        }
        $saveddoc = @$this->xmlDoc->save(DATAPATH . DS . $this->database);
        if($saveddoc == false){
            $returnvalue['error'] = isset($errorstring) ? $errorstring : 'There was problem saving xml document.';
            return $returnvalue;
        } else{
            $returnvalue['success'] = isset($successstring) ? $successstring : 'Data saved successfully.';
            return $returnvalue;
        }
    }

    /* Get total entries from the xml document.*/
    public function getTotalEntries(){
        $loadvalue = $this->loadXMLDoc();
        if($loadvalue == false){
            return 0;
        }
        $messageList = $this->xmlDoc->getElementsByTagName('message');
        $messageCount = $messageList->length;
        return $messageCount;
    }

    /* Add guestbook entry to xml file */
    public function addEntry($vals, $successmessage = null){
        $loadvalue = $this->loadXMLDoc();
        if($loadvalue == false){
            $returnvalue['error'] = "Error opening the xml database file. Your entry could not be added. Please contact and inform the guestbook owner.";
            return $returnvalue;
        }
        date_default_timezone_set('UTC');
        $timestamp = time();
        $fullname = ucwords(trim(strip_tags($vals[":fullname"])));
        $ip = $_SERVER ["REMOTE_ADDR"];
        $useragent = $_SERVER ["HTTP_USER_AGENT"];
        $emailid = trim(strip_tags($vals[":email"]));
        $website = trim(strip_tags($vals[":website"]));
        $comments = trim($vals[":comments"]);
        $this->badwords = Badwords::loadBadwords($this->badWordsFile);
        if($this->badwords) $comments = str_ireplace($this->badwords, "*****", $comments);
        $clean_comments = CommentPurifier::purify($comments);
        $messageid = $this->getRandomUniqueID();
        $guestbook = $this->xmlDoc->documentElement;
        $message = $this->xmlDoc->createElement("message");
        /* @var $messagenode \DOMElement */
        $messagenode = $guestbook->insertBefore($message, $guestbook->firstChild);
        $messagenode->setAttribute("uid", $messageid);
        $messagenode->setAttribute("datetime", $timestamp);
        $messagenode->appendChild($this->xmlDoc->createElement('ip', $ip));
        $messagenode->appendChild($this->xmlDoc->createElement('fullname', $fullname));
        $messagenode->appendChild($this->xmlDoc->createElement('email', $emailid));
        $messagenode->appendChild($this->xmlDoc->createElement('website', $website));
        $comnode = $this->xmlDoc->createElement('comments');
        $comnode->appendChild($this->xmlDoc->createCDATASection($clean_comments));
        $messagenode->appendChild($comnode);
        if($this->approval){
            $messagenode->appendChild($this->xmlDoc->createElement('approved', "false"));
        } else{
            $messagenode->appendChild($this->xmlDoc->createElement('approved', "true"));
        }
        $messagenode->appendChild($this->xmlDoc->createElement('useragent', $useragent));
        if(isset($successmessage)){
            $strSuccess = $successmessage[0] . (($this->approval) ? $successmessage[1] : "");
        } else {
            $strSuccess = "Entry added successfully. " . (($this->approval) ? "Please wait for the admin to approve." : "");
        }

        $result = $this->saveXMLDoc(null, $strSuccess);
        if(isset($result['success']) && $this->notify){
            $emailnotifier = new EmailNotifier();
            $emailnotifier->mail($this->adminemail, $messagenode, $this->timezone, $this->language);
        }
        return $result;
    }

    /* Build the guestbook entries pagewise with the admin template and display it.*/
    public function loadEntries($start, $limit, $all = false, $admin = false, $approved = "true"){
        $loadvalue = $this->loadXMLDoc();
        if($loadvalue == false){
            if($admin){
                return array('error' => 'Error opening the xml database file. Please check and make sure the database file <strong>' . $this->database . '</strong> is present in folder <strong>' . DATAPATH . '</strong>. Also make sure it is in valid xml format.');
            } else{
                return array('error' => 'Error opening the xml database file.');
            }
        }
        if($admin){
            $messageList = $this->xmlDoc->getElementsByTagName('message');
        } else{
            $approvedlist = new DOMXpath($this->xmlDoc);
            $messageList = $approvedlist->query("/guestbook/message[approved='$approved']");
        }
        $messageCount = $messageList->length;
        if($messageCount == 0) return array('empty' => 'No Entries Found!');
        $limit = $start + $limit;
        if($limit > $messageCount){
            $limit = $messageCount;
        }
        if($all){
            $start = 0;
            $limit = $messageCount;
        }
        $messagearray = [];
        for($i = $start; $i < $limit; $i++){
            /* @var $currentMessage \DOMElement */
            $currentMessage = $messageList->item($i);
            $uid = $currentMessage->getAttribute('uid');
            $timestamp = $currentMessage->getAttribute('datetime');
            $mdatetime = GBLocale::convertUTCTimetoTIMEZONETime($timestamp, $this->timezone, $this->language);
            $difftime = GBLocale::getHumanDiff($timestamp, $this->timezone, $this->language);
            $ip = $currentMessage->childNodes->item(0)->nodeValue;
            $fullname = $currentMessage->childNodes->item(1)->nodeValue;
            $emailid = $currentMessage->childNodes->item(2)->nodeValue;
            $website = $currentMessage->childNodes->item(3)->nodeValue;
            $comments = $currentMessage->childNodes->item(4)->nodeValue;
            $comments = $this->emoticon->parseSmileys($comments);
            $approved = $currentMessage->childNodes->item(5)->nodeValue == 'true' ? true : false;
            $grav_url = "http://www.gravatar.com/avatar/" . md5(strtolower($emailid)) . "?&s=60";
            $messagearray[$i]["uid"] = $uid;
            $messagearray[$i]["datetime"] = $mdatetime;
            $messagearray[$i]["difftime"] = $difftime;
            $messagearray[$i]["ip"] = $ip;
            $messagearray[$i]["fullname"] = $fullname;
            $messagearray[$i]["email"] = $emailid;
            $messagearray[$i]["website"] = $website;
            $messagearray[$i]["comments"] = $comments;
            $messagearray[$i]["approved"] = $approved;
            $messagearray[$i]["avatar"] = $grav_url;
        }
        return $messagearray;
    }

    public function updateXMLDoc(){
        $dbfile = DATAPATH . DS . $this->database;
        if(isset($_POST["submit"]) && isset($_POST["xmlcode"]) && !empty($_POST["xmlcode"])){
            if(file_put_contents($dbfile, $_POST["xmlcode"])){
                $returnvalue['success'] = "XML file successfully updated.";
                return $returnvalue;
            } else{
                $returnvalue['error'] = "Error saving the data to xml file. Please try again.";
                return $returnvalue;
            }
        } else{
            $returnvalue['error'] = "Error in data submitted. Please try again.";
            return $returnvalue;
        }
    }
}