<?php
namespace AJAXGuestBook\core;

use PHPMailer\PHPMailer\PHPMailer;
use PHPMailer\PHPMailer\Exception;

class EmailNotifier {
    private $fromaddress, $fromname, $toaddress, $subject, $body;

    public function mail($adminemail, $messagenode, $tz, $lang){
        $this->fromaddress = "postmaster@guestbook.com";
        $this->fromname = "GuestBook Admin";
        $this->toaddress = $adminemail;
        $this->subject = "New GuestBook Entry";
        $this->createBody($messagenode, $tz, $lang);
        $this->sendMail();
    }

    private function createBody($messagenode, $tz, $lang){
        /* @var $messagenode \DOMElement */
        $uid = $messagenode->getAttribute('uid');
        $mdatetime = $messagenode->getAttribute('datetime');
        $mdatetime = GBLocale::convertUTCTimetoTIMEZONETime($mdatetime, $tz, $lang);
        $ip = $messagenode->childNodes->item(0)->nodeValue;
        $fullname = $messagenode->childNodes->item(1)->nodeValue;
        $emailid = $messagenode->childNodes->item(2)->nodeValue;
        $website = $messagenode->childNodes->item(3)->nodeValue;
        $comments = $messagenode->childNodes->item(4)->nodeValue;
        $comments = nl2br(htmlspecialchars_decode($comments));
        $approved = $messagenode->childNodes->item(5)->nodeValue;
        $approved = ($approved === true ? "Approved" : "Pending");
        $useragent = $messagenode->childNodes->item(6)->nodeValue;
        $html = file_get_contents(VIEWS . DS . "bootstrap" . DS . "templates" . DS . "notifyemail.html");
        $html = str_replace("{{ip}}", $ip, $html);
        $html = str_replace("{{time}}", $mdatetime, $html);
        $html = str_replace("{{name}}", $fullname, $html);
        $html = str_replace("{{email}}", $emailid, $html);
        $html = str_replace("{{website}}", $website, $html);
        $html = str_replace("{{approved}}", $approved, $html);
        $html = str_replace("{{useragent}}", $useragent, $html);
        $html = str_replace("{{comment}}", $comments, $html);
        $url = isset($_SERVER['HTTPS']) ? 'https://' : 'http://';
        $url .= $_SERVER['SERVER_NAME'];
        $commenturl = $url . URL . "/admin/guestbook/edit/" . $uid;
        $html = str_replace("{{urllink}}", $commenturl, $html);
        $html = str_replace("{{urltext}}", $commenturl, $html);
        $this->body = $html;
    }

    private function sendMail(){
        $mail = new PHPMailer(true);
        try{
            $mail->setFrom($this->fromaddress, $this->fromname);
            $mail->addAddress($this->toaddress);
            $mail->isHTML(true);
            $mail->Subject = $this->subject;
            $mail->Body = $this->body;
            //$mail->AltBody = 'This is the body in plain text for non-HTML mail clients';
            $mail->send();
        } catch(Exception $e){
            $emailerror = ['error : ' => "Error sending email.", 'mesage : ' => $mail->ErrorInfo];
            Logger::log($emailerror);
        }
    }
}