<?php
namespace AJAXGuestBook\core;

class Emoticon {
    protected $folderpath, $folderurl, $emcodes, $emurls, $emlistfiles, $emsrc;

    function __construct($imagefolderpath, $imagefolderabsoluteurl){
        if(is_dir($imagefolderpath)){
            $this->folderpath = $imagefolderpath;
            $this->folderurl = $imagefolderabsoluteurl;
            $this->buildEmoticons();
        } else{
            $this->emcodes = $this->emurls = $this->emlistfiles = "Invalid folder";
        }
    }

    private function buildEmoticons(){
        $files = array_diff(scandir($this->folderpath), array('..', '.'));
        $this->emcodes = $this->emurls = $this->emlistfiles = $this->emsrc = [];
        if($files == false || !is_array($files)){
            return;
        }
        foreach($files as $file){
            if(!$this->checkImageType($this->folderpath . $file)) continue;
            $fileName = pathinfo($file, PATHINFO_FILENAME);
            array_push($this->emcodes, ":" . $fileName . ":");
            array_push($this->emurls, $this->folderurl . $file);
            array_push($this->emsrc, array('url' => $this->folderurl . $file, 'title' => $fileName));
            array_push($this->emlistfiles, $file);
        }
    }

    private function checkImageType($filename){
        if(function_exists('exif_imagetype')){
            $allowedTypes = array(IMAGETYPE_PNG, IMAGETYPE_JPEG, IMAGETYPE_GIF);
            $detectedType = @exif_imagetype($filename);
            return in_array($detectedType, $allowedTypes);
        } elseif(function_exists('getimagesize')){
            if(@is_array(getimagesize($filename))){
                return true;
            } else{
                return false;
            }
        } else{
            return false;
        }
    }

    public function getEmoticonCodeList(){
        return $this->emcodes;
    }

    public function getEmoticonUrlList(){
        return $this->emurls;
    }

    public function getEmoticonSrcList(){
        return $this->emsrc;
    }

    public function getEmoticonFileList(){
        return $this->emlistfiles;
    }

    public function parseSmileys($text){
        if(!is_array($this->emcodes) || !is_array($this->emurls)) return $text;
        $emoticons = array_combine($this->emcodes, $this->emurls);
        foreach($emoticons as $emotion => $imageurl){
            $text = str_replace($emotion, '<img class="gbsmiley" alt="' . $emotion . '" src="' . $imageurl . '" title="' . $emotion . '" />', $text);
        }
        return $text;
    }
}