<?php
namespace AJAXGuestBook\core;

use DOMDocument;
use DOMXPath;

/**
 * @property DOMDocument $xmlDoc
 */
class GuestBook extends Settings {
    use CommonMethods;
    public $database; //String: XML file name where all the comments are stored
    public $purifier; //Object: HTML Purifier class to filter the comments of message
    public $xmlDoc; //DOM Object: XML object to manipulate guestbook comments data
    public $badwords; //Array: To hold all the badwords
    public $badWordsFile; //String: File where all the badwors are stored
    public $emotions; //Array: Associative array holding the smileycode as key and respective image url as value
    public $emoticon;
    private static $instance = null;

    function __construct(){
        parent::__construct();
        $this->badWordsFile = "badwords.txt";
        $this->database = "data.xml";
        $emojipath = ROOTPATH . DS . 'public' . DS . 'img' . DS . 'smileys' . DS;
        $this->emoticon = new Emoticon($emojipath, URL . '/public/img/smileys/');
    }

    public static function getInstance(){
        if(self::$instance == null){
            self::$instance = new GuestBook();
        }
        return self::$instance;
    }

    /* Loading the xml document.*/
    private function loadXMLDoc(){
        if(!file_exists(DATAPATH . DS . $this->database)){
            return false;
        }
        $this->xmlDoc = new DOMDocument();
        $this->xmlDoc->validateOnParse = true;
        $this->xmlDoc->preserveWhiteSpace = false;
        $opened = $this->xmlDoc->Load(DATAPATH . DS . $this->database);
        if($opened === false){
            $xmlerror = libxml_get_last_error();
            libxml_clear_errors();
            Session::set("xmldberror", $xmlerror);
        } else{
            Session::delete("xmldberror");
        }
        return $opened;
    }

    /* Get total approved entries from the xml document.*/
    public function getApprovedEntries(){
        $loadvalue = $this->loadXMLDoc();
        if($loadvalue == false){
            return 0;
        }
        $approvedlist = new DOMXpath($this->xmlDoc);
        $messageList = $approvedlist->query("/guestbook/message[approved='true']");
        $messageCount = $messageList->length;
        return $messageCount;
    }

    /* Get total unapproved entries from the xml document.*/
    public function getUnapprovedEntries(){
        $loadvalue = $this->loadXMLDoc();
        if($loadvalue == false){
            return 0;
        }
        $approvedlist = new DOMXpath($this->xmlDoc);
        $messageList = $approvedlist->query("/guestbook/message[approved='false']");
        $messageCount = $messageList->length;
        return $messageCount;
    }

    /* Approve guestbook entry from xml file. */
    public function approveEntry($mid){
        $loadvalue = $this->loadXMLDoc();
        if($loadvalue == false){
            $returnvalue['error'] = 'Error opening the xml database file.';
            return $returnvalue;
        }
        $messages = new DOMXpath($this->xmlDoc);
        $messageList = $messages->query("/guestbook/message[@uid='" . $mid . "']");
        if($messageList->length == 1){
            $approve = $messageList->item(0);
        } else{
            $returnvalue['error'] = 'Error approving the guestbook entry.';
            return $returnvalue;
        }
        $status = $approve->childNodes->item(5);
        if($status == null){
            $approve->appendChild($this->xmlDoc->createElement("approved", "true"));
        } else{
            $status->nodeValue = "true";
        }
        return $this->saveXMLDoc(null, "Entry approved successfully.");
    }

    /* Load the entry details from the guestbook xml file to be edited or deleted. */
    public function loadGBEntry($mid){
        $loadvalue = $this->loadXMLDoc();
        if($loadvalue == false){
            $returnvalue['error'] = 'Error opening the xml database file.';
            return $returnvalue;
        }
        $messages = new DOMXpath($this->xmlDoc);
        $messageList = $messages->query("/guestbook/message[@uid='" . $mid . "']");
        if($messageList->length == 1){
            /* @var $edit \DOMElement */
            $edit = $messageList->item(0);
        } else{
            $returnvalue['error'] = 'Error loading the guestbook entry.';
            return $returnvalue;
        }
        $editdata['mid'] = $mid;
        $editdata['datetime'] = GBLocale::convertUTCTimetoTIMEZONETime($edit->getAttribute('datetime'), $this->timezone, $this->language);
        $editdata['ip'] = $edit->childNodes->item(0)->nodeValue;
        $editdata['fullname'] = $edit->childNodes->item(1)->nodeValue;
        $editdata['emailid'] = $edit->childNodes->item(2)->nodeValue;
        $editdata['website'] = $edit->childNodes->item(3)->nodeValue;
        $editdata['comments'] = $edit->childNodes->item(4)->nodeValue;
        $editdata['parsedcomments'] = $this->emoticon->parseSmileys($edit->childNodes->item(4)->nodeValue);
        $editdata['approved'] = $edit->childNodes->item(5)->nodeValue == 'true' ? true : false;
        $grav_url = "http://www.gravatar.com/avatar/" . md5(strtolower($editdata['emailid'])) . "?&s=60";
        $editdata["avatar"] = $grav_url;
        $returnvalue['data'] = json_encode($editdata);
        return $returnvalue;
    }

    //TODO load selected entries for deletion and display to confirm
    /*public function loadSelectedEntries(){

    }*/
    /* Delete guestbook entry from xml file. */
    public function deleteGBEntry(){
        $mid = $_POST ['deleteentry'];
        if(GBDEMO && $mid == "mid011687g123456"){
            $returnvalue['error'] = 'Demo entry showing guestbook features list cannot be deleted.';
            return $returnvalue;
        }
        $loadvalue = $this->loadXMLDoc();
        if($loadvalue == false){
            $returnvalue['error'] = 'Error opening the xml database file.';
            return $returnvalue;
        }
        $root = $this->xmlDoc->documentElement;
        $messages = new DOMXpath($this->xmlDoc);
        $messageList = $messages->query("/guestbook/message[@uid='" . $mid . "']");
        if($messageList->length == 1){
            $remove = $messageList->item(0);
        } else{
            $returnvalue['error'] = 'Error deleting the entry from database.';
            return $returnvalue;
        }
        $root->removeChild($remove);
        return $this->saveXMLDoc(null, "Entry deleted successfully.");
    }

    /* Delete multiple guestbook entries from xml file. */
    public function deleteMultipleGBEntries(){
        $loadvalue = $this->loadXMLDoc();
        if($loadvalue == false){
            $returnvalue['error'] = "Error opening the xml database file.";
            return $returnvalue;
        }
        $rmcheck = false;
        $root = $this->xmlDoc->documentElement;
        $messages = new DOMXpath($this->xmlDoc);
        if(is_array($_POST['cbmid'])){
            foreach($_POST['cbmid'] as $value){
                if(GBDEMO && $value == "mid011687g123456"){
                    Session::set("gberror", "Demo entry showing guestbook features list cannot be deleted.");
                    continue;
                }
                $messageList = $messages->query("/guestbook/message[@uid='" . $value . "']");
                if($messageList->length == 1){
                    $remove = $messageList->item(0);
                    $root->removeChild($remove);
                    $rmcheck = true;
                }
            }
        }
        if(!$rmcheck){
            $returnvalue['success'] = "Please select other entries and not demo.";
            return $returnvalue;
        }
        return $this->saveXMLDoc(null, "Entries deleted successfully.");
    }

    /* Send the edited entry details of guestbook entry and update the xml file. */
    public function updateGBEntry(){
        $mid = $_POST ['updateentry'];
        if(isset($_POST ['cbApproved'])){
            $approved = "true";
        } else{
            $approved = "false";
        }
        if(GBDEMO && $mid == "mid011687g123456"){
            $returnvalue['error'] = 'This entry shows the guestbook features and cannot be edited.';
            return $returnvalue;
        }
        $loadvalue = $this->loadXMLDoc();
        if($loadvalue == false){
            $returnvalue['error'] = 'Error opening the xml database file.';
            return $returnvalue;
        }
        $messages = new DOMXpath($this->xmlDoc);
        $messageList = $messages->query("/guestbook/message[@uid='" . $mid . "']");
        if($messageList->length == 1){
            $editnode = $messageList->item(0);
        } else{
            $returnvalue['error'] = 'Error updating guestbook entry.';
            return $returnvalue;
        }
        $fullname = ucwords(trim(strip_tags($_POST ['fullname'])));
        $emailid = trim(strip_tags($_POST ['email']));
        $website = trim(strip_tags($_POST ['website']));
        $comments = trim($_POST ['comments']);
        $this->badwords = Badwords::loadBadwords($this->badWordsFile);
        if($this->badwords){
            $comments = str_ireplace($this->badwords, "*****", $comments);
        }
        $clean_comments = CommentPurifier::purify($comments);
        $editnode->childNodes->item(1)->nodeValue = $fullname;
        $editnode->childNodes->item(2)->nodeValue = $emailid;
        $editnode->childNodes->item(3)->nodeValue = $website;
        $comnode = $this->xmlDoc->createElement('comments');
        $comnode->appendChild($this->xmlDoc->createCDATASection($clean_comments));
        $editnode->replaceChild($comnode, $editnode->childNodes->item(4));
        //$editnode->childNodes->item(4)->nodeValue = $clean_comments;
        $editnode->childNodes->item(5)->nodeValue = $approved;
        return $this->saveXMLDoc(null, "Entry updated successfully.");
    }

    public function rectifySpam($message){
        $loadvalue = $this->loadXMLDoc();
        if($loadvalue == false){
            $returnvalue['error'] = "Error opening the xml database file.";
            return $returnvalue;
        }
        /* @var $importmessage \DOMElement
         * @var $message \DOMElement
         */
        $importmessage = $this->xmlDoc->importNode($message, true);
        $messageid = $this->getRandomUniqueID();
        $importmessage->setAttribute("uid", $messageid);
        $guestbook = $this->xmlDoc->documentElement;
        $messages = new DOMXpath($this->xmlDoc);
        $datetime = $message->getAttribute('datetime');
        $messageList = $messages->query("/guestbook/message[@datetime<'" . $datetime . "'][1]");
        if($messageList->length == 1){
            $insertbefore = $messageList->item(0);
            $guestbook->insertBefore($importmessage, $insertbefore);
        } else{
            $returnvalue['error'] = "Error saving spam entry.";
            return $returnvalue;
        }
        return $this->saveXMLDoc(null, "Entry approved successfully.");
    }
}