<?php
namespace AJAXGuestBook\core;

class Paginator {
    var $items_per_page;
    var $items_total;
    var $current_page;
    var $total_pages;
    var $mid_range;
    var $low;
    var $high;
    var $limit;
    var $default_ipp;
    var $querystring;
    var $ipp;
    var $url;
    var $start_range;
    var $end_range;
    var $range;
    var $firstpage;
    var $lastpage;
    var $pagelinks;
    var $prevpageurl;
    var $nextpageurl;

    function __construct(){
        $this->current_page = 1;
        $this->mid_range = 7;
        $this->default_ipp = 10;
    }

    function paginate($url){
        $this->url = $url;
        if(isset($_GET['ipp'])){
            if(strtolower($_GET['ipp']) == 'all'){
                $this->ipp = "All";
            } else{
                $this->ipp = (int)$_GET['ipp'];
            }
        } else{
            $this->ipp = $this->default_ipp;
        }
        if(!is_numeric($this->items_total) OR $this->items_total <= 0) return;
        $this->items_per_page = (!empty($this->ipp)) ? $this->ipp : $this->default_ipp;
        if(isset($this->ipp) AND $this->ipp == 'All'){
            $this->total_pages = ceil($this->items_total / $this->default_ipp);
            $this->items_per_page = $this->default_ipp;
        } else{
            if(!is_numeric($this->items_per_page) OR $this->items_per_page <= 0) $this->items_per_page = $this->default_ipp;
            $this->total_pages = ceil($this->items_total / $this->items_per_page);
        }
        $this->current_page = isset($_GET['page']) ? (int)$_GET['page'] : 1; // must be numeric > 0
        if($this->current_page < 1 Or !is_numeric($this->current_page)) $this->current_page = 1;
        if($this->current_page > $this->total_pages) $this->current_page = $this->total_pages;
        $prev_page = $this->current_page - 1;
        $next_page = $this->current_page + 1;
        if($this->total_pages > 10){
            ($this->current_page != 1 And $this->items_total >= 10) ? $this->prevpageurl = "$this->url?&page=$prev_page&ipp=$this->items_per_page" : $this->firstpage = true;
            $this->start_range = $this->current_page - floor($this->mid_range / 2);
            $this->end_range = $this->current_page + floor($this->mid_range / 2);
            if($this->start_range <= 0){
                $this->end_range += abs($this->start_range) + 1;
                $this->start_range = 1;
            }
            if($this->end_range > $this->total_pages){
                $this->start_range -= $this->end_range - $this->total_pages;
                $this->end_range = $this->total_pages;
            }
            $this->range = range($this->start_range, $this->end_range);
            for($i = 1; $i <= $this->total_pages; $i++){
                if($this->range[0] > 2 And $i == $this->range[0]) $this->pagelinks[$i] = '...';
                // loop through all pages. if first, last, or in range, display
                if($i == 1 Or $i == $this->total_pages Or in_array($i, $this->range)){
                    if($i == $this->current_page And $this->ipp != 'All'){
                        $this->pagelinks[$i] = array('active' => true, 'url' => "#", 'pagenum' => $i);
                    } else{
                        $this->pagelinks[$i] = array('url' => "$this->url?&page=$i&ipp=$this->items_per_page",
                                                     'pagenum' => $i);
                    }
                }
                if($this->range[$this->mid_range - 1] < $this->total_pages - 1 And $i == $this->range[$this->mid_range - 1]) $this->pagelinks[$i] = '...';
            }
            if($this->current_page != $this->total_pages And $this->items_total >= 10){
                $this->nextpageurl = "$this->url?&page=$next_page&ipp=$this->items_per_page";
            } else{
                $this->lastpage = true;
            }
        } else{
            for($i = 1; $i <= $this->total_pages; $i++){
                if($i == $this->current_page And $this->ipp != 'All'){
                    $this->pagelinks[$i] = array('active' => true, 'url' => "#", 'pagenum' => $i);
                } else{
                    $this->pagelinks[$i] = array('url' => "$this->url?&page=$i&ipp=$this->items_per_page",
                                                 'pagenum' => $i);
                }
            }
        }
        $this->low = ($this->current_page - 1) * $this->items_per_page;
        $this->high = ($this->ipp == 'All') ? $this->items_total : ($this->current_page * $this->items_per_page) - 1;
        $this->limit = ($this->ipp == 'All') ? "" : " LIMIT $this->low,$this->items_per_page";
    }

    function page_data(){
        return ['onfirstpage' => $this->firstpage, 'prevpageurl' => $this->prevpageurl, 'onlastpage' => $this->lastpage,
                'nextpageurl' => $this->nextpageurl, 'pagelinks' => $this->pagelinks, 'itemsperpage' => $this->items_per_page,
                'totalpages' => $this->total_pages, 'currentpage' => $this->current_page];
    }
}