<?php
namespace AJAXGuestBook\core;

use DOMDocument;

class Settings {
    public $adminid; //String: Administrator login id
    public $pass; //String: Administrator password
    public $adminemail; //String: Email address of the admin to notify about new entry
    public $notify; //Boolean: To check whether to notify admin about new entry
    public $theme; //String: Guestbook theme
    public $title; //String: Guestbook title
    public $company; //String: Company using the guestbook
    public $website; //String: Domain URL of the company
    public $captcha; //Boolean: Captcha requirements for posting entry to guestbook
    public $approval; //Boolean: to check whether new guestbook entry should be awaited for approval
    public $requiredFields; //Array: Storing the compulsory required field for new guestbook entry
    public $minCommentsChars, $maxCommentsChars; //Integer: Minimum and maximum characters allowence in the comments
    public $entriesPerPage; //Integer: Number of comments to be displayed per page
    public $timezone, $language; //String: Timezone and language to display date accordingly
    public $apikey, $enableakismet; //String and Boolean: Akismet is to prevent spam
    private $settingsDoc;

    function __construct(){
        $this->settingsDoc = new DOMDocument();
        $this->settingsDoc->preserveWhiteSpace = false;
        $this->settingsDoc->formatOutput = true;
        $loadvalue = @$this->settingsDoc->Load(DATAPATH . DS . "settings.xml");
        if($loadvalue == false){
            exit(settingsError());
        }
        $this->loadSettings();
    }

    public function loadSettings(){
        $settinglist = $this->settingsDoc->documentElement;
        $gbook = $settinglist->getElementsByTagName('guestbook')->item(0);
        $this->title = $gbook->childNodes->item(0)->nodeValue;
        $this->company = $gbook->childNodes->item(1)->nodeValue;
        $this->website = $gbook->childNodes->item(2)->nodeValue;
        $gbadmin = $settinglist->getElementsByTagName('admin')->item(0);
        $this->adminid = $gbadmin->childNodes->item(0)->nodeValue;
        $this->pass = $gbadmin->childNodes->item(1)->nodeValue;
        $this->adminemail = $gbadmin->childNodes->item(2)->nodeValue;
        $this->notify = intval($gbadmin->childNodes->item(3)->nodeValue);
        $this->theme = $settinglist->getElementsByTagName('theme')->item(0)->nodeValue;
        $this->captcha = $settinglist->getElementsByTagName('captcha')->item(0)->nodeValue == 'true' ? true : false;
        $gbrfield = $settinglist->getElementsByTagName('requiredfields')->item(0);
        $this->requiredFields['gb_name'] = intval($gbrfield->childNodes->item(0)->nodeValue);
        $this->requiredFields['gb_email'] = intval($gbrfield->childNodes->item(1)->nodeValue);
        $this->requiredFields['gb_website'] = intval($gbrfield->childNodes->item(2)->nodeValue);
        $this->requiredFields['gb_comments'] = intval($gbrfield->childNodes->item(3)->nodeValue);
        $gbcomments = $settinglist->getElementsByTagName('comments')->item(0);
        $this->minCommentsChars = intval($gbcomments->childNodes->item(0)->nodeValue);
        $this->maxCommentsChars = intval($gbcomments->childNodes->item(1)->nodeValue);
        $this->entriesPerPage = intval($gbcomments->childNodes->item(2)->nodeValue);
        $this->approval = $settinglist->getElementsByTagName('approval')->item(0)->nodeValue == 'true' ? true : false;
        $gblocale = $settinglist->getElementsByTagName('locale')->item(0);
        if($gblocale){
            $this->timezone = $gblocale->childNodes->item(0)->nodeValue;
            $this->language = $gblocale->childNodes->item(1)->nodeValue;
        }
        $gbakismet = $settinglist->getElementsByTagName('akismet')->item(0);
        if($gbakismet){
            $this->enableakismet = intval($gbakismet->childNodes->item(0)->nodeValue);
            $this->apikey = $gbakismet->childNodes->item(1)->nodeValue;
        }
    }

    /* Save general settings to the xml document.*/
    public function saveGeneralSettings($vals){
        $root = $this->settingsDoc->documentElement;
        $replaceguestbook = $root->getElementsByTagName('guestbook')->item(0);
        $replaceguestbook->childNodes->item(0)->nodeValue = $vals[':gbtitle'];
        $replaceguestbook->childNodes->item(1)->nodeValue = $vals[':gbcompany'];
        $replaceguestbook->childNodes->item(2)->nodeValue = $vals[':gbwebsite'];
        $replacerequiredfields = $root->getElementsByTagName('requiredfields')->item(0);
        $replacerequiredfields->childNodes->item(0)->nodeValue = $vals[':gbreqname'];
        $replacerequiredfields->childNodes->item(1)->nodeValue = $vals[':gbreqemail'];
        $replacerequiredfields->childNodes->item(2)->nodeValue = $vals[':gbreqweb'];
        $replacerequiredfields->childNodes->item(3)->nodeValue = $vals[':gbreqcomment'];
        $replacecaptcha = $root->getElementsByTagName('captcha')->item(0);
        $captchaEL = $this->settingsDoc->createElement("captcha", $vals[':gbrequirecaptcha']);
        if($replacecaptcha == null){
            $root->appendChild($captchaEL);
        } else{
            $root->replaceChild($captchaEL, $replacecaptcha);
        }
        $replacecomments = $root->getElementsByTagName('comments')->item(0);
        $replacecomments->childNodes->item(0)->nodeValue = $vals[':gbcommentsminchar'];
        $replacecomments->childNodes->item(1)->nodeValue = $vals[':gbcommentsmaxchar'];
        $replacecomments->childNodes->item(2)->nodeValue = $vals[':gbentriesperpage'];
        $replaceapproval = $root->getElementsByTagName('approval')->item(0);
        $approvalEL = $this->settingsDoc->createElement("approval", $vals[':gbapproval']);
        if($replaceapproval == null){
            $root->appendChild($approvalEL);
        } else{
            $root->replaceChild($approvalEL, $replaceapproval);
        }
        return $this->saveSettingsDoc(null, 'General settings updated successfully.');
    }

    /* Loading Admin profile settings from the xml document.*/
    public function loadProfileSettings($profiletype){
        $profile = [];
        switch($profiletype){
            case "loadid":
                $profile['admin']['id'] = $this->adminid;
                break;
            case "loadpass":
                $profile['admin']['pass'] = $this->pass;
                break;
            case "loademail":
                $profile['admin']['email'] = $this->adminemail;
                break;
            case "loadprofile":
                $profile['admin']['id'] = $this->adminid;
                $profile['admin']['email'] = $this->adminemail;
                $profile['admin']['notify'] = $this->notify;
                break;
            default:
                break;
        }
        return $profile;
    }

    /* Save Admin profile settings to the xml document.*/
    public function saveProfileSettings($vals){
        $root = $this->settingsDoc->documentElement;
        $adminel = $root->getElementsByTagName('admin')->item(0);
        $profiletype = $vals["pofiletype"];
        switch($profiletype){
            case "setid":
                $adminel->childNodes->item(0)->nodeValue = $vals["newadminid"];
                break;
            case "setpassword":
                $adminel->childNodes->item(1)->nodeValue = $vals["newadminpass"];
                break;
            case "setemail":
                $adminel->childNodes->item(2)->nodeValue = $vals["newadminemail"];
                $adminel->childNodes->item(3)->nodeValue = $vals["notifyadmin"];
                break;
            default:
                break;
        }
        return $this->saveSettingsDoc(null, 'Profile settings updated successfully.');
    }

    /* Save locale settings to the xml document.*/
    public function saveLocaleSettings($vals){
        $root = $this->settingsDoc->documentElement;
        $localeel = $root->getElementsByTagName('locale')->item(0);
        if($localeel == null){
            $localeel = $this->settingsDoc->createElement("locale");
            $localeel->appendChild($this->settingsDoc->createElement("timezone", $vals["timezone"]));
            $localeel->appendChild($this->settingsDoc->createElement("language", $vals["language"]));
            $root->appendChild($localeel);
        } else{
            $localeel->childNodes->item(0)->nodeValue = $vals["timezone"];
            $localeel->childNodes->item(1)->nodeValue = $vals["language"];
        }
        return $this->saveSettingsDoc(null, 'Locale settings updated successfully.');
    }

    /* Save theme settings to the xml document.*/
    public function saveThemeSettings($vals){

        if(!is_dir(VIEWS.DS.$vals['theme'])){
            Session::set('themeerror', 'Please select a valid theme.');
            $returnvalue['error'] = 'Selected theme does not exists.';
            return $returnvalue;
           }

        $root = $this->settingsDoc->documentElement;
        $themeel = $root->getElementsByTagName('theme')->item(0);
        if($themeel == null){
            $themeel = $this->settingsDoc->createElement("theme", $vals["theme"]);
            $root->appendChild($themeel);
        } else{
            $themeel->nodeValue = $vals["theme"];
        }
        return $this->saveSettingsDoc(null, 'Theme settings updated successfully.');
    }

    /* Save spam related settings to the xml document.*/
    public function saveSpamSettings($vals){
        $root = $this->settingsDoc->documentElement;
        $akismetel = $root->getElementsByTagName('akismet')->item(0);
        if($akismetel == null){
            $akismetel = $this->settingsDoc->createElement("akismet");
            $akismetel->appendChild($this->settingsDoc->createElement("enable", $vals["enableakismet"]));
            $akismetel->appendChild($this->settingsDoc->createElement("apikey", $vals["apikey"]));
            $root->appendChild($akismetel);
        } else{
            $akismetel->childNodes->item(0)->nodeValue = $vals["enableakismet"];
            $akismetel->childNodes->item(1)->nodeValue = $vals["apikey"];
        }
        return $this->saveSettingsDoc(null, 'Akismet settings updated successfully.');
    }

    private function saveSettingsDoc($errorstring = null, $successstring = null){
        $saved = $this->settingsDoc->save(DATAPATH . DS . "settings.xml");
        if($saved == false){
            $returnvalue['error'] = isset($errorstring) ? $errorstring : 'There was problem saving settings. Please try again.';
            return $returnvalue;
        } else{
            $this->loadSettings();
            $returnvalue['success'] = isset($successstring) ? $successstring : 'Guestbook settings updated successfully.';
            return $returnvalue;
        }
    }

    public function getSettings(){
        return array('title' => $this->title, 'company' => $this->company, 'website' => $this->website,
                     'adminid' => $this->adminid, 'pass' => $this->pass, 'adminemail' => $this->adminemail,
                     'notify' => $this->notify, 'theme' => $this->theme, 'captcha' => $this->captcha,
                     'requiredFields' => $this->requiredFields, 'minCommentsChars' => $this->minCommentsChars,
                     'maxCommentsChars' => $this->maxCommentsChars, 'entriesPerPage' => $this->entriesPerPage,
                     'approval' => $this->approval, 'timezone' => $this->timezone, 'language' => $this->language,
                     'enableakismet' => $this->enableakismet, 'apikey' => $this->apikey,);
    }
}