<?php
namespace AJAXGuestBook\core;

use DOMDocument;
use DOMXPath;

/**
 * @property DOMDocument $xmlDoc
 */
class SpamBook extends Settings {
    use CommonMethods;
    public $database; //String: XML file name where all the comments are stored
    public $purifier; //Object: HTML Purifier class to filter the comments of message
    public $xmlDoc; //DOM Object: XML object to manipulate guestbook comments data
    public $badwords; //Array: To hold all the badwords
    public $badWordsFile; //String: File where all the badwors are stored
    public $emotions; //Array: Associative array holding the smileycode as key and respective image url as value
    public $emoticon;
    private static $instance = null;

    function __construct(){
        parent::__construct();
        $this->badWordsFile = "badwords.txt";
        $this->database = "spamdata.xml";
        $emojipath = ROOTPATH . DS . 'public' . DS . 'img' . DS . 'smileys' . DS;
        $this->emoticon = new Emoticon($emojipath, URL . '/public/img/smileys/');
    }

    public static function getInstance(){
        if(self::$instance == null){
            self::$instance = new SpamBook();
        }
        return self::$instance;
    }

    /* Loading the spam xml document.*/
    private function loadXMLDoc(){
        if(!file_exists(DATAPATH . DS . $this->database)){
            $spamDoc = new DOMDocument('1.0');
            $spamDoc->preserveWhiteSpace = false;
            $guestbook = $spamDoc->createElement("guestbook");
            $spamDoc->appendChild($guestbook);
            $saved = $spamDoc->save(DATAPATH . DS . $this->database);
            unset($spamDoc);
            if($saved == false){
                Session::set("gberror", 'There was problem creating database file. Please contact developer.');
                return false;
            }
        }
        $this->xmlDoc = new DOMDocument();
        $this->xmlDoc->validateOnParse = true;
        $this->xmlDoc->preserveWhiteSpace = false;
        $opened = @$this->xmlDoc->Load(DATAPATH . DS . $this->database);
        if($opened === false){
            $xmlerror = libxml_get_last_error();
            libxml_clear_errors();
            Session::set("xmldberror", $xmlerror);
        } else{
            Session::delete("xmldberror");
        }
        return $opened;
    }

    /* Approve spam entry from xml file. */
    public function approveSpamEntry($mid){
        $loadvalue = $this->loadXMLDoc();
        if($loadvalue == false){
            $returnvalue['error'] = "Error opening the xml database file.";
            return $returnvalue;
        }
        $root = $this->xmlDoc->documentElement;
        $messages = new DOMXpath($this->xmlDoc);
        $messageList = $messages->query("/guestbook/message[@uid='" . $mid . "']");
        if($messageList->length == 1){
            $approve = $messageList->item(0);
        } else{
            $returnvalue['error'] = "Error approving entry.";
            return $returnvalue;
        }
        $status = $approve->childNodes->item(5);
        if($status == null){
            $approve->appendChild($this->xmlDoc->createElement("approved", "true"));
        } else{
            $status->nodeValue = "true";
        }
        $gb = GuestBook::getInstance();
        $result = $gb->rectifySpam($approve);
        if($result['error']) return $result;
        $root->removeChild($approve);
        return $this->saveXMLDoc("Entry added to Guestbook. But there was problem removing entry from spam database.", "Entry approved successfully and added to guestbook.");
    }

    /* Delete spam entry from xml file. */
    public function deleteSpamEntry($mid){
        $loadvalue = $this->loadXMLDoc();
        if($loadvalue == false){
            $returnvalue['error'] = "Error opening the xml database file.";
            return $returnvalue;
        }
        $root = $this->xmlDoc->documentElement;
        $messages = new DOMXpath($this->xmlDoc);
        $messageList = $messages->query("/guestbook/message[@uid='" . $mid . "']");
        if($messageList->length == 1){
            $remove = $messageList->item(0);
        } else{
            $returnvalue['error'] = "Error deleting entry from database.";
            return $returnvalue;
        }
        $root->removeChild($remove);
        return $this->saveXMLDoc(null, "Entry deleted successfully.");
    }

    /* Delete multiple spam entries from xml file. */
    public function deleteMultipleGBEntries(){
        $loadvalue = $this->loadXMLDoc();
        if($loadvalue == false){
            $returnvalue['error'] = "Error opening the xml database file.";
            return $returnvalue;
        }
        $root = $this->xmlDoc->documentElement;
        $messages = new DOMXpath($this->xmlDoc);
        if(is_array($_POST['cbmid'])){
            foreach($_POST['cbmid'] as $value){
                $messageList = $messages->query("/guestbook/message[@uid='" . $value . "']");
                if($messageList->length == 1){
                    $remove = $messageList->item(0);
                    $root->removeChild($remove);
                }
            }
        }
        return $this->saveXMLDoc(null, "Entries deleted successfully.");
    }
}