<?php
namespace AJAXGuestBook\core;

use Twig\Loader\FilesystemLoader;
use Twig\Environment;
use Twig\Extension\DebugExtension;

/**
 * @property Environment $twig
 */
class View {
    protected $default_template_dir = 'bootstrap';
    protected $template_dir;
    protected $twig;

    public function __construct($theme = null){
        if($theme !== null){
            $this->template_dir = $theme;
        }
        $this->initializeTwig();
    }

    private function initializeTwig(){
        $loader = new FilesystemLoader([$this->template_dir, $this->default_template_dir, VIEWS], VIEWS);
        $this->twig = new Environment($loader, ['debug' => true]);

        /*$loader = new FilesystemLoader([],VIEWS);
        $loader->addPath($this->default_template_dir, 'default');
        $loader->prependPath($this->template_dir);*/

        $this->twig = new Environment($loader, ['debug' => true]);
        $this->twig->addExtension(new GuestbookBookExtension());
        $this->twig->addExtension(new DebugExtension());
    }

    public function render($template_file, $vars){
        try{
            return $this->twig->render($template_file, $vars);
        } catch(\Exception $e){
            print_r('<pre>'.$e.'</pre>');
            die("Template not found.");
        }
    }

    public function addGlobals($item, $value){
        $this->twig->addGlobal($item, $value);
    }

    public function addTwigFunction($funcname, $caller, $options){
        $function = new \Twig\TwigFunction($funcname, $caller, $options);
        $this->twig->addFunction($function);
    }
}