<?php
namespace AJAXGuestBook\models;

use AJAXGuestBook\core\GuestBook;
use AJAXGuestBook\core\Model;
use AJAXGuestBook\core\Session;
use AJAXGuestBook\core\Validation;
use Akismet;
use PasswordHash;

/**
 * @property GuestBook $guestbook
 */
class AdminModel extends Model {
    private $formdata;
    private $passhash;
    private $guestbook;

    function __construct($gbook){
        parent::__construct();
        $this->guestbook = $gbook;
    }

    public function updateGeneralSettings(){
        if(isset($_POST['submit'])){
            if(isset($_POST['savegbsett'])){
                $result = $this->validateGeneralSettings();
                if($result === false){
                    $returnvalue['error'] = 'Please correct the errors below marked in red background with error message.';
                    return $returnvalue;
                } else{
                    return $this->guestbook->saveGeneralSettings($this->formdata);
                }
            } else{
                $returnvalue['error'] = 'Some error occured. Please try again.';
                return $returnvalue;
            }
        } else{
            $returnvalue['error'] = 'Some error occured. Please try again.';
            return $returnvalue;
        }
    }

    private function validateGeneralSettings(){
        $error = false;
        array_filter($_POST, array($this, 'trim_value'));
        if(isset($_POST["gbtitle"])){
            $valResult = Validation::validateTitle($_POST["gbtitle"]);
            if($valResult === true){
                $gbtitle = filter_input(INPUT_POST, "gbtitle", FILTER_SANITIZE_STRING);
            } else{
                $error = true;
                Session::set("titleerror", $valResult);
            }
        } else{
            $error = true;
            Session::set("titleerror", "Title cannot be blank.");
        }
        if(isset($_POST["gbcompany"])){
            $valResult = Validation::validateCompany($_POST["gbcompany"]);
            if($valResult === true){
                $gbcompany = filter_input(INPUT_POST, "gbcompany", FILTER_SANITIZE_STRING);
            } else{
                $error = true;
                Session::set("companyerror", $valResult);
            }
        } else{
            $error = true;
            Session::set("companyerror", "Company name cannot be blank.");
        }
        if(isset($_POST["gbwebsite"])){
            $valResult = Validation::validateCompanyUrl($_POST["gbwebsite"]);
            if($valResult === true){
                $gbwebsite = filter_input(INPUT_POST, "gbwebsite", FILTER_SANITIZE_STRING);
            } else{
                $error = true;
                Session::set("urlerror", $valResult);
            }
        } else{
            $error = true;
            Session::set("urlerror", "Company url cannot be blank.");
        }
        if(isset($_POST["captcha"])){
            $gbcap = $_POST ['captcha'];
            $gbrequirecaptcha = ($gbcap == 'On') ? 'true' : 'false';
        } else{
            $gbrequirecaptcha = 'false';
        }
        if(isset($_POST ['cbName'])){
            $gbreqname = 1;
        } else{
            $gbreqname = 0;
        }
        if(isset($_POST ['cbEmail'])){
            $gbreqemail = 1;
        } else{
            $gbreqemail = 0;
        }
        if(isset($_POST ['cbWebsite'])){
            $gbreqweb = 1;
        } else{
            $gbreqweb = 0;
        }
        if(isset($_POST ['cbComments'])){
            $gbreqcomment = 1;
        } else{
            $gbreqcomment = 0;
        }
        if(isset($_POST["approval"])){
            $gbrequireapproval = $_POST ['approval'];
            $gbapproval = ($gbrequireapproval == 'On') ? 'true' : 'false';
        } else{
            $gbapproval = 'false';
        }
        if(isset($_POST["txtMinchar"])){
            $valResult = Validation::validateInteger($_POST["txtMinchar"], "Minimum Characters ");
            if($valResult === true){
                $gbcommentsminchar = filter_input(INPUT_POST, "txtMinchar", FILTER_SANITIZE_NUMBER_INT);
            } else{
                $error = true;
                Session::set("gbcommentsmincharerror", $valResult);
            }
        } else{
            $error = true;
            Session::set("gbcommentsmincharerror", "Minimum Characters cannot be blank.");
        }
        if(isset($_POST["txtMaxchar"])){
            $valResult = Validation::validateInteger($_POST["txtMaxchar"], "Maximum Characters ");
            if($valResult === true){
                $gbcommentsmaxchar = filter_input(INPUT_POST, "txtMaxchar", FILTER_SANITIZE_NUMBER_INT);
                if($gbcommentsmaxchar != 0 && $gbcommentsmaxchar < $gbcommentsminchar){
                    $error = true;
                    Session::set("gbcommentsmaxcharerror", "Maximum Characters should be greater than Minumum Characters.");
                }
            } else{
                $error = true;
                Session::set("gbcommentsmaxcharerror", $valResult);
            }
        } else{
            $error = true;
            Session::set("gbcommentsmaxcharerror", "Maximum Characters cannot be blank. Set it to 0 for unlimited.");
        }
        if(isset($_POST["txtEntriesPerpage"])){
            $valResult = Validation::validateInteger($_POST["txtEntriesPerpage"], "Entries per page ");
            if($valResult === true){
                $gbentriesperpage = filter_input(INPUT_POST, "txtEntriesPerpage", FILTER_SANITIZE_NUMBER_INT);
            } else{
                $error = true;
                Session::set("gbentriesperpageerror", $valResult);
            }
        } else{
            $error = true;
            Session::set("gbentriesperpageerror", "Entries per page cannot be blank.");
        }
        if($error){
            return false;
        } else{
            $this->formdata = array(':gbtitle' => $gbtitle, ':gbcompany' => $gbcompany, ':gbwebsite' => $gbwebsite,
                                    ':gbrequirecaptcha' => $gbrequirecaptcha, ':gbreqname' => $gbreqname,
                                    ':gbreqemail' => $gbreqemail, ':gbreqweb' => $gbreqweb,
                                    ':gbreqcomment' => $gbreqcomment, ':gbapproval' => $gbapproval,
                                    ':gbcommentsminchar' => $gbcommentsminchar,
                                    ':gbcommentsmaxchar' => $gbcommentsmaxchar,
                                    ':gbentriesperpage' => $gbentriesperpage);
            return true;
        }
    }

    public function updateProfileSettings(){
        if(isset($_POST['submit'])){
            if(isset($_POST['updateproile'])){
                $profiletype = $_POST['updateproile'];
                $result = $this->validateProfileSettings($profiletype);
                if($result === false){
                    $returnvalue['error'] = 'Please correct the errors below marked in red background with error message.';
                    return $returnvalue;
                } else{
                    return $this->guestbook->saveProfileSettings($this->formdata);
                }
            } else{
                $returnvalue['error'] = 'Some error occured. Please try again.';
                return $returnvalue;
            }
        } else{
            $returnvalue['error'] = 'Some error occured. Please try again.';
            return $returnvalue;
        }
    }

    private function validateProfileSettings($profiletype){
        $error = false;
        array_filter($_POST, array($this, 'trim_value'));
        switch($profiletype){
            case "setid":
                if(isset($_POST["txtID"])){
                    $valResult = Validation::validateAdminID($_POST["txtID"]);
                    if($valResult === true){
                        $newadminid = filter_input(INPUT_POST, "txtID", FILTER_SANITIZE_STRING);
                        if($this->checkAdminIdExists($newadminid)){
                            $error = true;
                            Session::set("adminiderror", "Admin Id already exists. Please choose a different one.");
                        }
                    } else{
                        $error = true;
                        Session::set("adminiderror", $valResult);
                    }
                } else{
                    $error = true;
                    Session::set("adminiderror", "Admin ID cannot be blank.");
                }
                if(isset($_POST["txtPassword"])){
                    $valResult = Validation::validatePassword($_POST["txtPassword"]);
                    if($valResult === true){
                        $this->passhash = new PasswordHash(8, true);
                        if(!$this->passhash->CheckPassword($_POST["txtPassword"], $this->guestbook->pass)){
                            $error = true;
                            Session::set("passerror", "Password entered is invalid.");
                        }
                    } else{
                        $error = true;
                        Session::set("passerror", $valResult);
                    }
                } else{
                    $error = true;
                    Session::set("passerror", "Password cannot be blank.");
                }
                if($error == false) $this->formdata = array('pofiletype' => "setid", 'newadminid' => $newadminid);
                break;
            case "setpassword":
                if(isset($_POST["txtNewPassword"]) and isset($_POST["txtConfirmPassword"])){
                    $valResult = Validation::validatePassword($_POST["txtNewPassword"]);
                    if($valResult === true){
                        if($_POST["txtNewPassword"] != $_POST["txtConfirmPassword"]){
                            $error = true;
                            Session::set("newpasserror", "Password does not match.");
                        } else{
                            $newpassword = $_POST["txtNewPassword"];
                        }
                    } else{
                        $error = true;
                        Session::set("newpasserror", $valResult);
                    }
                } else{
                    $error = true;
                    Session::set("newpasserror", "New Password or Confirm Password cannot be blank.");
                }
                if(isset($_POST["txtCurrentPassword"])){
                    $valResult = Validation::validatePassword($_POST["txtCurrentPassword"]);
                    if($valResult === true){
                        if($error === false){
                            $oldpassword = $_POST["txtCurrentPassword"];
                            $currentpassword = $this->guestbook->loadProfileSettings("loadpass");
                            $this->passhash = new PasswordHash(8, true);
                            if($this->passhash->CheckPassword($oldpassword, $currentpassword['admin']['pass'])){
                                if($this->passhash->CheckPassword($newpassword, $currentpassword['admin']['pass'])){
                                    $error = true;
                                    Session::set("newpasserror", "New password should be different than old password.");
                                } else{
                                    $newpassword = $this->passhash->HashPassword($newpassword);
                                }
                            } else{
                                $error = true;
                                Session::set("currentpasserror", "Current password entry is invalid.");
                            }
                        }
                    } else{
                        $error = true;
                        Session::set("currentpasserror", $valResult);
                    }
                } else{
                    $error = true;
                    Session::set("currentpasserror", "Password cannot be blank.");
                }
                if($error == false) $this->formdata = array('pofiletype' => "setpassword",
                                                            'newadminpass' => $newpassword);
                break;
            case "setemail":
                if(isset($_POST["txtAdminemail"])){
                    $valResult = Validation::validateEmail($_POST["txtAdminemail"], 'Email');
                    if($valResult === true){
                        $email = filter_input(INPUT_POST, "txtAdminemail", FILTER_SANITIZE_STRING);
                    } else{
                        $error = true;
                        Session::set("adminemailerror", $valResult);
                    }
                } else{
                    $error = true;
                    Session::set("adminemailerror", "Email address cannot be blank.");
                }
                if(isset($_POST["txtEmailPassword"])){
                    $valResult = Validation::validatePassword($_POST["txtEmailPassword"]);
                    if($valResult === true){
                        $currentpassword = $this->guestbook->loadProfileSettings("loadpass");
                        $this->passhash = new PasswordHash(8, true);
                        if(!$this->passhash->CheckPassword($_POST["txtEmailPassword"], $currentpassword['admin']['pass'])){
                            $error = true;
                            Session::set("emailpasserror", "Password entered is invalid.");
                        }
                    } else{
                        $error = true;
                        Session::set("emailpasserror", $valResult);
                    }
                } else{
                    $error = true;
                    Session::set("emailpasserror", "Password cannot be blank.");
                }
                $notifyadmin = (isset($_POST ['cbAdminEmail'])) ? 1 : 0;
                if($error == false) $this->formdata = array('pofiletype' => "setemail", 'newadminemail' => $email,
                                                            'notifyadmin' => $notifyadmin);
                break;
            default:
                Session::set("gberror", "Some error occured. Please try again.");
                break;
        }
        if($error){
            return false;
        } else{
            return true;
        }
    }

    private function checkAdminIdExists($newadminid){
        $oldid = $this->guestbook->loadProfileSettings("loadid");
        return ($newadminid == $oldid['admin']['id']);
    }

    public function updateSpamSettings(){
        if(isset($_POST['updatespam'])){
            array_filter($_POST, array($this, 'trim_value'));
            $vals["enableakismet"] = (isset($_POST['cbAPIkey']) ? 1 : 0);
            if($_POST["txtAPIkey"] === "" && $vals["enableakismet"] === 1){
                Session::set("akismetkeyerror", "Please enter Akismet API key or obtain a valid one from https://akismet.com");
                $returnvalue['error'] = 'Please correct the errors below marked in red background with error message.';
                return $returnvalue;
            }
            $vals["apikey"] = $_POST["txtAPIkey"];
            if($vals["apikey"] != ""){
                if($this->guestbook->apikey != $vals["apikey"]){
                    $akismet = new Akismet(appURL(), $vals["apikey"]);
                    if(!$akismet->checkAPIKey()){
                        Session::set("akismetkeyerror", "The Akismet API key is invalid. Please obtain a valid one from https://akismet.com");
                        $returnvalue['error'] = 'Please correct the errors below marked in red background with error message.';
                        return $returnvalue;
                    }
                }
            }
            if($this->guestbook->enableakismet === $vals["enableakismet"] && $this->guestbook->apikey == $vals["apikey"]){
                $returnvalue['error'] = 'Form data is not changed.';
                return $returnvalue;
            } else{
                return $this->guestbook->saveSpamSettings($vals);
            }
        } else{
            $returnvalue['error'] = 'Some error occured. Please try again.';
            return $returnvalue;
        }
    }
}
