<?php
namespace AJAXGuestBook\models;

use AJAXGuestBook\core\GBLocale;
use AJAXGuestBook\core\GuestBook;
use AJAXGuestBook\core\Model;
use AJAXGuestBook\core\Session;
use AJAXGuestBook\core\SpamBook;
use AJAXGuestBook\core\Validation;
use DOMDocument;
use DOMXpath;
use Akismet;

/**
 * @property GuestBook $guestbook
 */
class EntriesModel extends Model {
    private $guestbook;
    private $langVars;

    function __construct($gbook){
        parent::__construct();
        $this->guestbook = $gbook;
        $this->langVars = GBLocale::getLanguageVariables(['FormSignUp','ErrorMessages','SuccessMessages']);
    }

    public function addEntry(){
        if(isset($_POST['addentry'])){
            $successmessage = array($this->langVars["SuccessMessages"]["entryAdded"],$this->langVars["SuccessMessages"]["entryApproval"]);
            if($this->validateEntryData() === false){
                $returnvalue['error'] = $this->langVars["ErrorMessages"]["alert"];
                return $returnvalue;
            } else{
                if($this->guestbook->enableakismet == true){
                    $akismet = new Akismet(appURL(), $this->guestbook->apikey);
                    if($akismet->checkAPIKey()){
                        $akismet->setCommentAuthor($this->commentsdata[":fullname"])->setCommentAuthorEmail($this->commentsdata[":email"])->setCommentContent($this->commentsdata[":comments"]);
                        if($akismet->isCommentSpam()){
                            $spambook = SpamBook::getInstance();
                            return $spambook->addEntry($this->commentsdata, $successmessage);
                        }
                    }
                }
                return $this->guestbook->addEntry($this->commentsdata, $successmessage);
            }
        } else{
            $returnvalue['error'] = "No valid post data found.";
            return $returnvalue;
        }
    }

    private function convertSmileys($comment){
        $html = new DOMDocument();
        $html->loadHTML('<div id="gbcode">' . $comment . '</div>');
        $xpath = new DOMXpath($html);
        /* @var $images \DOMElement
         * @var $image \DOMElement
         */
        $images = $xpath->query('//img[@class="gbsmiley"]');
        if($images->length > 0){
            foreach($images as $image){
                $alttext = $image->getAttribute('alt');
                $smileycode = $html->createTextNode($alttext);
                $image->parentNode->replaceChild($smileycode, $image);
            }
            $rawhtml = $html->saveHTML($html->getElementById("gbcode"));
            $rawhtml = substr($rawhtml, 15, -4);
            return $rawhtml;
        } else{
            return $comment;
        }
    }

    public function validateEntryData(){
        $error = false;
        array_filter($_POST, array($this, 'trim_value'));
        if(isset($_POST["fullname"]) AND !empty($_POST['fullname']) Or $this->guestbook->requiredFields["gb_name"] == 1){
            $valResult = Validation::validateName($_POST["fullname"], $this->langVars["FormSignUp"]["FieldName"]);
            if($valResult === true){
                $fullname = filter_input(INPUT_POST, "fullname", FILTER_SANITIZE_STRING);
                $fullname = ucfirst(strtolower($fullname));
            } else{
                $error = true;
                Session::set("fullnameerror", $valResult);
            }
        } else {
            $fullname = "Anonymous";
        }
        if(isset($_POST["email"]) and !empty($_POST['email']) or $this->guestbook->requiredFields["gb_email"] == 1){
            $valResult = Validation::validateEmail($_POST["email"], $this->langVars["FormSignUp"]["FieldEmail"]);
            if($valResult === true){
                $email = filter_input(INPUT_POST, "email", FILTER_SANITIZE_STRING);
            } else{
                $error = true;
                Session::set("emailerror", $valResult);
            }
        } else{
            $email = "";
        }
        if(isset($_POST["website"]) and !empty($_POST['website']) or $this->guestbook->requiredFields["gb_website"] == 1){
            $valResult = Validation::validateUrl($_POST["website"], $this->langVars["FormSignUp"]["FieldWebsite"]);
            if($valResult === true){
                $website = filter_input(INPUT_POST, "website", FILTER_SANITIZE_STRING);
            } else{
                $error = true;
                Session::set("websiteerror", $valResult);
            }
        } else{
            $website = "";
        }
        if(isset($_POST['comments']) and !empty($_POST['comments']) or $this->guestbook->requiredFields["gb_comments"] == 1){
            if($this->guestbook->maxCommentsChars == 0) $this->guestbook->maxCommentsChars = null;
            $comments = $_POST['comments'];
            $comments = $this->convertSmileys($comments);
            $valResult = Validation::validateComments($comments, $this->guestbook->minCommentsChars,
                $this->guestbook->maxCommentsChars, $this->langVars["FormSignUp"]["FieldComments"]);
            if($valResult !== true){
                $error = true;
                Session::set("commentserror", $valResult);
            }
        } else{
            $comments = "";
        }
        if($this->guestbook->captcha){
            $captchacode = (isset($_POST['captchacode']) ? $_POST['captchacode'] : null);
            $valResult = Validation::validateCaptcha($captchacode, $_SESSION ['captchacode'], $this->langVars["FormSignUp"]["FieldCaptcha"]);
            if($valResult !== true){
                $error = true;
                Session::set("captchaerror", $valResult);
            }
        }
        if($error){
            return false;
        } else{
            $this->commentsdata = array(':fullname' => $fullname, ':email' => $email, ':website' => $website,
                                        ':comments' => $comments);
            return true;
        }
    }
}