<?php
namespace AJAXGuestBook\models;

use AJAXGuestBook\core\GBLocale;
use AJAXGuestBook\core\Model;
use AJAXGuestBook\core\Session;
use AJAXGuestBook\core\Validation;
use DOMDocument;
use PasswordHash;

class InstallModel extends Model {
    function __construct(){
        parent::__construct();
    }

    public function validateGeneralSettings(){
        $error = false;
        Session::delete('installgeneral');
        array_filter($_POST, array($this, 'trim_value'));
        if(isset($_POST["gbtitle"]) && $_POST["gbtitle"] != ""){
            $valResult = Validation::validateTitle($_POST["gbtitle"]);
            if($valResult === true){
                $gbtitle = filter_input(INPUT_POST, "gbtitle", FILTER_SANITIZE_STRING);
            } else{
                $error = true;
                Session::set("titleerror", $valResult);
            }
        } else{
            $error = true;
            Session::set("titleerror", "Title cannot be blank.");
        }
        if(isset($_POST["gbcompany"]) && $_POST["gbcompany"] != ""){
            $valResult = Validation::validateCompany($_POST["gbcompany"]);
            if($valResult === true){
                $gbcompany = filter_input(INPUT_POST, "gbcompany", FILTER_SANITIZE_STRING);
            } else{
                $error = true;
                Session::set("companyerror", $valResult);
            }
        } else{
            $error = true;
            Session::set("companyerror", "Company name cannot be blank.");
        }
        if(isset($_POST["gbwebsite"]) && $_POST["gbwebsite"] != ""){
            $valResult = Validation::validateCompanyUrl($_POST["gbwebsite"]);
            if($valResult === true){
                $gbwebsite = filter_input(INPUT_POST, "gbwebsite", FILTER_SANITIZE_STRING);
            } else{
                $error = true;
                Session::set("urlerror", $valResult);
            }
        } else{
            $error = true;
            Session::set("urlerror", "Company url cannot be blank.");
        }
        if(isset($_POST ['cbName'])){
            $gbreqname = 1;
        } else{
            $gbreqname = 0;
        }
        if(isset($_POST ['cbEmail'])){
            $gbreqemail = 1;
        } else{
            $gbreqemail = 0;
        }
        if(isset($_POST ['cbWebsite'])){
            $gbreqweb = 1;
        } else{
            $gbreqweb = 0;
        }
        if(isset($_POST ['cbComments'])){
            $gbreqcomment = 1;
        } else{
            $gbreqcomment = 0;
        }
        if(isset($_POST["captcha"])){
            $gbrequirecaptcha = ($_POST ['captcha'] == 'On') ? 'true' : 'false';
        } else{
            $gbrequirecaptcha = 'false';
        }
        if(isset($_POST["approval"])){
            $gbapproval = ($_POST ['approval'] == 'On') ? 'true' : 'false';
        } else{
            $gbapproval = 'false';
        }
        if(isset($_POST["txtMinchar"]) && $_POST["txtMinchar"] != ""){
            $valResult = Validation::validateInteger($_POST["txtMinchar"], "Minimum Characters ");
            if($valResult === true){
                $gbcommentsminchar = filter_input(INPUT_POST, "txtMinchar", FILTER_SANITIZE_NUMBER_INT);
            } else{
                $error = true;
                Session::set("gbcommentsmincharerror", $valResult);
            }
        } else{
            $error = true;
            Session::set("gbcommentsmincharerror", "Minimum Characters cannot be blank.");
        }
        if(isset($_POST["txtMaxchar"]) && $_POST["txtMaxchar"] != ""){
            $valResult = Validation::validateInteger($_POST["txtMaxchar"], "Maximum Characters ");
            if($valResult === true){
                $gbcommentsmaxchar = filter_input(INPUT_POST, "txtMaxchar", FILTER_SANITIZE_NUMBER_INT);
                if($gbcommentsmaxchar != 0 && $gbcommentsmaxchar < $gbcommentsminchar){
                    $error = true;
                    Session::set("gbcommentsmaxcharerror", "Maximum Characters should be greater than Minumum Characters.");
                }
            } else{
                $error = true;
                Session::set("gbcommentsmaxcharerror", $valResult);
            }
        } else{
            $error = true;
            Session::set("gbcommentsmaxcharerror", "Maximum Characters cannot be blank. Set it to 0 for unlimited.");
        }
        if(isset($_POST["txtEntriesPerpage"]) && $_POST["txtEntriesPerpage"] != ""){
            $valResult = Validation::validateInteger($_POST["txtEntriesPerpage"], "Entries per page ");
            if($valResult === true){
                $gbentriesperpage = filter_input(INPUT_POST, "txtEntriesPerpage", FILTER_SANITIZE_NUMBER_INT);
            } else{
                $error = true;
                Session::set("gbentriesperpageerror", $valResult);
            }
        } else{
            $error = true;
            Session::set("gbentriesperpageerror", "Entries per page cannot be blank.");
        }
        if($error){
            return false;
        } else{
            $installgeneral = array(':gbtitle' => $gbtitle, ':gbcompany' => $gbcompany, ':gbwebsite' => $gbwebsite,
                                    ':gbrequirecaptcha' => $gbrequirecaptcha, ':gbreqname' => $gbreqname,
                                    ':gbreqemail' => $gbreqemail, ':gbreqweb' => $gbreqweb,
                                    ':gbreqcomment' => $gbreqcomment, ':gbapproval' => $gbapproval,
                                    ':gbcommentsminchar' => $gbcommentsminchar,
                                    ':gbcommentsmaxchar' => $gbcommentsmaxchar,
                                    ':gbentriesperpage' => $gbentriesperpage);
            Session::set("installgeneral", $installgeneral);
            return true;
        }
    }

    public function validateProfileSettings(){
        $error = false;
        Session::delete('installprofile');
        array_filter($_POST, array($this, 'trim_value'));
        if(isset($_POST["txtID"]) && $_POST["txtID"] != ""){
            $valResult = Validation::validateAdminID($_POST["txtID"]);
            if($valResult === true){
                $adminid = filter_input(INPUT_POST, "txtID", FILTER_SANITIZE_STRING);
            } else{
                $error = true;
                Session::set("adminiderror", $valResult);
            }
        } else{
            $error = true;
            Session::set("adminiderror", "Admin ID cannot be blank.");
        }
        if(isset($_POST["txtPassword"]) and isset($_POST["txtConfirmPassword"])){
            $valResult = Validation::validatePassword($_POST["txtPassword"]);
            if($valResult === true){
                if($_POST["txtPassword"] != $_POST["txtConfirmPassword"]){
                    $error = true;
                    Session::set("passerror", "Password does not match.");
                } else{
                    $password = $_POST["txtPassword"];
                    $passhash = new PasswordHash(8, true);
                    $hashedpassword = $passhash->HashPassword($password);
                }
            } else{
                $error = true;
                Session::set("passerror", $valResult);
            }
        } else{
            $error = true;
            Session::set("passerror", "Password or Confirm Password cannot be blank.");
        }
        if(isset($_POST["txtAdminemail"])){
            $valResult = Validation::validateEmail($_POST["txtAdminemail"], 'Email');
            if($valResult === true){
                $email = filter_input(INPUT_POST, "txtAdminemail", FILTER_SANITIZE_STRING);
            } else{
                $error = true;
                Session::set("adminemailerror", $valResult);
            }
        } else{
            $error = true;
            Session::set("adminemailerror", "Email address cannot be blank.");
        }
        if(isset($_POST ['cbAdminEmail'])){
            $notifyadmin = 1;
        } else{
            $notifyadmin = 0;
        }
        if($error){
            return false;
        } else{
            $installprofile = array(':adminid' => $adminid, ':plainpass' => $password, ':adminpass' => $hashedpassword,
                                    ':adminemail' => $email, ':notifyadmin' => $notifyadmin);
            Session::set("installprofile", $installprofile);
            return true;
        }
    }

    public function validateLocaleSettings(){
        $error = false;
        Session::delete('installlocale');
        $timezones = GBLocale::getTimeZoneList();
        $languages = GBLocale::getCarbonLocales();
        array_filter($_POST, array($this, 'trim_value'));
        if(isset($_POST["timezone"])){
            $gbtz = filter_input(INPUT_POST, "timezone", FILTER_SANITIZE_STRING);
            if(array_search($gbtz, $timezones) === false){
                $error = true;
                Session::set("timezoneerror", "Please select a valid timezone.");
            }
        } else{
            $error = true;
            Session::set("timezoneerror", "Please select a valid timezone.");
        }
        if(isset($_POST["language"])){
            $gblang = filter_input(INPUT_POST, "language", FILTER_SANITIZE_STRING);
            if(array_key_exists($gblang, $languages) === false){
                $error = true;
                Session::set("languageerror", "Please select a valid language.");
            }
        } else{
            $error = true;
            Session::set("languageerror", "Please select a valid language.");
        }
        if($error){
            return false;
        } else{
            $installlocale = array(':timezone' => $gbtz, ':language' => $gblang);
            Session::set("installlocale", $installlocale);
            return true;
        }
    }

    public function installSettings(){
        $installgeneral = Session::get('installgeneral');
        $installprofile = Session::get('installprofile');
        $installlocale = Session::get('installlocale');
        $settingsxmlDoc = new DOMDocument('1.0', 'utf-8');
        $settingsxmlDoc->preserveWhiteSpace = false;
        $settingsxmlDoc->formatOutput = true;
        $root = $settingsxmlDoc->createElement('settings');
        $settingsxmlDoc->appendChild($root);
        $guestbook = $settingsxmlDoc->createElement('guestbook');
        $root->appendChild($guestbook);
        $guestbook->appendChild($settingsxmlDoc->createElement('title', $installgeneral[':gbtitle']));
        $guestbook->appendChild($settingsxmlDoc->createElement('company', $installgeneral[':gbcompany']));
        $guestbook->appendChild($settingsxmlDoc->createElement('website', $installgeneral[':gbwebsite']));
        $admin = $settingsxmlDoc->createElement('admin');
        $root->appendChild($admin);
        $admin->appendChild($settingsxmlDoc->createElement('id', $installprofile[':adminid']));
        $admin->appendChild($settingsxmlDoc->createElement('pass', $installprofile[':adminpass']));
        $admin->appendChild($settingsxmlDoc->createElement('email', $installprofile[':adminemail']));
        $admin->appendChild($settingsxmlDoc->createElement('notify', $installprofile[':notifyadmin']));
        $root->appendChild($settingsxmlDoc->createElement('theme', 'bootstrap'));
        $root->appendChild($settingsxmlDoc->createElement('captcha', $installgeneral[':gbrequirecaptcha']));
        $requiredfields = $settingsxmlDoc->createElement('requiredfields');
        $root->appendChild($requiredfields);
        $requiredfields->appendChild($settingsxmlDoc->createElement('name', $installgeneral[':gbreqname']));
        $requiredfields->appendChild($settingsxmlDoc->createElement('email', $installgeneral[':gbreqemail']));
        $requiredfields->appendChild($settingsxmlDoc->createElement('web', $installgeneral[':gbreqweb']));
        $requiredfields->appendChild($settingsxmlDoc->createElement('comment', $installgeneral[':gbreqcomment']));
        $comments = $settingsxmlDoc->createElement('comments');
        $root->appendChild($comments);
        $comments->appendChild($settingsxmlDoc->createElement('minchars', $installgeneral[':gbcommentsminchar']));
        $comments->appendChild($settingsxmlDoc->createElement('maxchars', $installgeneral[':gbcommentsmaxchar']));
        $comments->appendChild($settingsxmlDoc->createElement('perpage', $installgeneral[':gbentriesperpage']));
        $root->appendChild($settingsxmlDoc->createElement('approval', $installgeneral[':gbapproval']));
        $locale = $settingsxmlDoc->createElement('locale');
        $root->appendChild($locale);
        $locale->appendChild($settingsxmlDoc->createElement('timezone', $installlocale[':timezone']));
        $locale->appendChild($settingsxmlDoc->createElement('language', $installlocale[':language']));
        $saved = $settingsxmlDoc->save(DATAPATH . DS . "settings.xml");
        if($saved == false){
            Session::set("gberror", "Problem saving the settings. Please try again.");
            return false;
        } else{
            Session::set("gbinfo", "Guestbook settings saved successfully.");
            $result = $this->createhtaccessAndInstallLock();
            if($result) return false;
            Session::clear();
            Session::set("gbinfo", "Guestbook settings saved successfully. Also created htaccess and locked the installation.");
            if(!file_exists(DATAPATH . "/data.xml")){
                $this->CreateSampleData();
            } else{
                Session::set("olddbexists", true);
            }
            return true;
        }
    }

    private function createhtaccessAndInstallLock(){
        $error = false;
        $htaccess = <<<HTDATA
# prevent reading .txt and .xml files
<Files ~ "\.(txt|xml)$">
    Deny From All
</Files>
HTDATA;
        $htaccessFH = @fopen(DATAPATH . DS . ".htaccess", "w");
        if($htaccessFH === false){
            Session::set("gberror", 'Error saving .htaccess file. Please contact developer.');
            $error = true;
        } else{
            fwrite($htaccessFH, $htaccess);
            fclose($htaccessFH);
        }
        $installlockFH = @fopen(DATAPATH . DS . "install.lock", "w");
        if($installlockFH === false){
            Session::set("gberror", 'Error saving install.lock file. Please contact developer.');
            $error = true;
        } else{
            fwrite($installlockFH, "Guestbook Installed");
            fclose($installlockFH);
        }
        return $error;
    }

    private function CreateSampleData(){
        date_default_timezone_set('UTC');
        $timestamp = time();
        $fullname = 'AigleMedia';
        $ip = $_SERVER ["REMOTE_ADDR"];
        $useragent = $_SERVER ["HTTP_USER_AGENT"];
        $emailid = 'admin@aiglemedia.com';
        $website = 'http://www.aiglemedia.com';
        $messageid = 'mid011687g123456';
        $comments = <<<COMMENTDATA
<span style="color:#FF0000;font-size:20px;">Guestbook owner can delete this comment by logging on to admin area.</span><br>
<strong>This is just a test entry to show supported features.</strong>
<ol><li><strong>Bold</strong>, <em>Italics</em>, <u>Underline</u>, <del>Strike</del> text.</li>
    <li>You can write <sub>Sub Script</sub>, and <sup>Super Script</sup> text</li>
    <li><span style="font-size:17px;">Different </span><span style="font-size:13px;">text </span><span style="font-size:35px;">size</span></li>
    <li>Make it <span style="font-size:35px;"><span style="color:#FF0000;">C</span><span style="color:#008000;">O</span><span style="color:#FFA500;">L</span><span style="color:#0000FF;">OR</span><span style="color:#FF0000;">F</span><span style="color:#008000;">U</span><span style="color:#FFA500;">L</span></span> text.</li>
    <li><ul><li>Unordered</li><li>list</li></ul></li>
    <li><ol><li>Ordered</li><li>list</li></ol></li>
    <li><blockquote>This is a blockquote with link to <a href="http://www.aiglemedia.com" target="_blank">aiglemedia.com</a></blockquote></li>
    <li>You can add images also<br> <img alt="" src="https://source.unsplash.com/random/400x300"></li>
</ol>
COMMENTDATA;
        $xmlDoc = new DOMDocument('1.0');
        $xmlDoc->preserveWhiteSpace = false;
        $guestbook = $xmlDoc->createElement("guestbook");
        $guestbook->setAttribute('dbversion', DATABASE_VERSION);
        $xmlDoc->appendChild($guestbook);
        $message = $xmlDoc->createElement("message");
        /* @var $messagenode \DOMElement */
        $messagenode = $guestbook->insertBefore($message, $guestbook->firstChild);
        $messagenode->setAttribute("uid", $messageid);
        $messagenode->setAttribute("datetime", $timestamp);
        $messagenode->appendChild($xmlDoc->createElement('ip', $ip));
        $messagenode->appendChild($xmlDoc->createElement('fullname', $fullname));
        $messagenode->appendChild($xmlDoc->createElement('email', $emailid));
        $messagenode->appendChild($xmlDoc->createElement('website', $website));
        $comnode = $xmlDoc->createElement('comments');
        $comnode->appendChild($xmlDoc->createCDATASection($comments));
        $messagenode->appendChild($comnode);
        $messagenode->appendChild($xmlDoc->createElement('approved', "true"));
        $messagenode->appendChild($xmlDoc->createElement('useragent', $useragent));
        $saved = $xmlDoc->save(DATAPATH . DS . "data.xml");
        if($saved == false){
            Session::set("gberror", 'There was problem creating database file. Please contact developer.');
            return false;
        } else{
            Session::set("gbsuccess", 'Your guestbook was installed successfully.');
            return true;
        }
    }
}